/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module uart_rx(
    clk      ,
    rst_n    ,
    din      ,
    dout     ,
    dout_vld
    );

parameter          DATA_W = 8    ;
parameter          NUM_W  = 4    ;
parameter          CNT_W  = 14   ;
parameter    	   BPS	  =	5208 ;	
parameter 	       BPS_P  = BPS/2;	

input               clk          ;		
input               rst_n        ;	
input               din          ;
output[DATA_W-1:0]  dout         ;		
output              dout_vld     ;

reg   [DATA_W-1:0]  dout         ;
reg                 dout_vld     ;
reg   [NUM_W-1 :0]  data_num     ;
reg   [DATA_W-1:0]  rx_temp_data ;
reg                 din_ff0      ;	
reg                 din_ff1      ;	
reg                 din_ff2      ;	
reg                 flag_add     ;
wire                end_cnt      ;
wire                end_cnt_p    ;
wire                add_data_num ;
wire                end_data_num ;

reg   [CNT_W-1:0]   cnt          ;	
wire                add_cnt      ;
always @ (posedge clk or negedge rst_n) begin
	if(!rst_n) begin
		din_ff0 <= 1'b1;
        din_ff1 <= 1'b1;
        din_ff2 <= 1'b1;
	end
	else begin
		din_ff0 <= din;
        din_ff1 <= din_ff0;
        din_ff2 <= din_ff1;
	end
end

always @ (posedge clk or negedge rst_n)begin
	if(!rst_n) begin
		flag_add <= 1'b0;
	end
	else if(din_ff2 & ~din_ff1) begin		
		flag_add <= 1'b1;	
	end
	else if(data_num==4'd8&&end_cnt) begin		
		flag_add <= 1'b0;	
	end
end

always @ (posedge clk or negedge rst_n)begin
    if(!rst_n)begin
        cnt <= 0;
    end
    else if(add_cnt)begin
        if(end_cnt)begin
            cnt <= 0;	 
        end
        else begin
            cnt <= cnt+1'b1; 
        end
    end
    else begin
        cnt <= 0; 
    end
end
assign add_cnt = flag_add;
assign end_cnt = add_cnt && cnt == BPS-1;



always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        data_num <= 0; 
    end
    else if(add_data_num) begin
        if(end_data_num)
            data_num <= 0; 
        else
            data_num <= data_num+1 ;
   end
end
assign add_data_num = end_cnt;
assign end_data_num = add_data_num  && data_num == 9-1 ;

always @ (posedge clk or negedge rst_n)begin
	if(!rst_n) begin
		dout <= 8'd0;
	end
	else if(add_cnt && cnt==BPS_P-1 && data_num!=0) begin		
	    dout<={din,{dout[7:1]}};
	end
    else begin
        dout<=dout; 
    end
end

always @ (posedge clk or negedge rst_n)begin
	if(!rst_n) begin
		dout_vld <= 1'b0;
	end
    else if(add_data_num && data_num == 4'd8) begin		
		dout_vld <= 1'b1;	
	end
	else begin	
        dout_vld <= 1'b0;			
	end
end
endmodule

